@@ -1,3 +1,4 @@
+from threading import local
 import traceback
 from function_scheduling_distributed_framework import fsdf_background_scheduler, task_deco, patch_frame_config, get_publisher, BrokerEnum
 from function_scheduling_distributed_framework.consumers.base_consumer import ExceptionForRequeue
@@ -65,7 +66,7 @@ def on_mqtt_message(client, userdata, msg):
     
 minio = MinioAgent('{}:{}'.format(opt.host, opt.port), opt.acs_key, opt.acs_secret, secure=False, location=opt.location)
 
-cache_engine = CacheManager(opt.redis_host, opt.redis_pass, opt.redis_port, opt.redis_db)
+cache_engine = CacheManager(opt.redis_host, opt.redis_pass, opt.redis_port, opt.redis_db, local_cache=None)
 
 mqtt_client = MQTTClient('fs_server', opt.mqtt_user, opt.mqtt_pass).connect(opt.mqtt_host, opt.mqtt_port)
 mqtt_client.subscribe('fs_backend_update', 2)
@@ -136,53 +137,45 @@ async def file_transfer_put(url, bucket, bucket_name, ttl):
 
 
 @task_deco('fs_get', function_timeout=60, concurrent_mode=4, broker_kind=BrokerEnum.NSQ)
-async def file_transfer_get(bucket, bucket_name):
-    key = 'file_transfer_get_{}_{}'.format(bucket, bucket_name)
-    if await cache_engine.get_cache_async(key, return_obj=False) is not None:
-        return
-    
-    with cache_engine.lock(key, ttl=30) as lock:
-        if lock:
-            if await cache_engine.get_cache_async(key, return_obj=False) is not None:
-                return
-            file_path = os.path.join(cache_path, '{}_{}'.format(bucket, bucket_name))
-            if not os.path.isfile(file_path):
-                ret, e = minio.download_file(file_path, bucket, bucket_name)
-                if ret:
-                    file_path = e
-                else:
-                    cache_engine.cache('tmp_file_{}_{}'.format(bucket, bucket_name), {
-                        'ret': False, 
-                        'err': e.message
-                    }, expired=60)
-                    
-                    cache_engine.cache(key, 1, expired=60)
-                    print('ok')
-                    return
-
-            # uploading
-            remote_state = await cache_engine.get_cache_async('tmp_file_{}_{}'.format(bucket, bucket_name))
+async def file_transfer_get(bucket, bucket_name, refresh='no'):
+    try:
+        task_id = 'tmp_file_{}_{}'.format(bucket, bucket_name)
+        if refresh == 'no':
+            remote_state = await cache_engine.get_cache_async(task_id)
             if remote_state is not None:
                 ret = remote_state['ret']
                 if ret:
+                    print('remote has record->{}, ignored.'.format(task_id))
                     return
-                
-            ret, e = hmfs_client.put_to_cloud(file_path, bucket, bucket_name) #await file_manager.upload_file(file_path,'tmp_file_' + bucket, bucket_name, False, backend='qiniu')
+
+        file_path = os.path.join(cache_path, '{}_{}'.format(bucket, bucket_name))
+        if not os.path.isfile(file_path):
+            ret, e = minio.download_file(file_path, bucket, bucket_name)
             if ret:
-                cache_engine.cache('tmp_file_{}_{}'.format(bucket, bucket_name), {
-                    'ret': True,
-                    'url': e
-                }, expired=60 * 60 * 24 * 1)
-                print('upload success!!')
+                file_path = e
             else:
-                cache_engine.cache('tmp_file_{}_{}'.format(bucket, bucket_name), {
+                cache_engine.cache(task_id, {
                     'ret': False, 
-                    'err': e
+                    'err': e.message
                 }, expired=60)
-            cache_engine.cache(key, 1, expired=60)
-            print('ok')
-            
-            
+                return
+
+        ret, e = hmfs_client.put_to_cloud(file_path, bucket, bucket_name)
+        if ret:
+            cache_engine.cache(task_id, {
+                'ret': True,
+                'url': e
+            }, expired=60 * 60 * 24 * 1)
+            print('upload success!!')
+        else:
+            cache_engine.cache(task_id, {
+                'ret': False, 
+                'err': e
+            }, expired=60)
+    except Exception as e:
+        traceback.print_exc()
+    
+
 # @task_deco('file_transfer_del', function_timeout=60, concurrent_mode=4, broker_kind=BrokerEnum.NSQ)
 # async def file_transfer_del(bucket, bucket_name):
 #     key = 'del_file_{}_{}'.format(bucket, bucket_name)
